# -*- coding: utf-8 -*-

from django.test import tag

from accounts.models import User, Group
from accounts.utils import update_user, get_ldap_user
from accounts.backends import AccountsCASBackend
from ldapaccounts.models import (
    LdapUser,
    LdapGroup,
    LdapClub,
    LdapPosixGroup,
)

from guichet.tests import CustomTestCase
from .utils import test_user, createLdapUser
from django_cas_ng.signals import cas_user_authenticated
from django.core.exceptions import SuspiciousOperation


class ModelCoverage(CustomTestCase):
    @tag("fast", "nologin")
    def test_create_superuser(self):
        superuser = User.objects.create_superuser("admin@billetterie.inpt.fr", "admin", "admin", "admin")
        self.assertTrue(superuser.is_superuser)

    @tag("fast", "nologin")
    def test_anonymeuser(self):

        anonyme_user = User.get_anonyme_user()
        anonyme2_user = User.get_anonyme_user()
        self.assertEqual(anonyme_user, anonyme2_user)
        self.assertTrue(anonyme_user.is_anonyme)
        self.assertFalse(anonyme_user.is_active)
        self.assertEqual(anonyme_user.get_full_name(), "Anonyme")
        self.assertEqual(anonyme_user.get_short_name(), "Anonyme")
        self.assertEqual(str(anonyme_user), "Anonyme")
        with self.assertRaises(SuspiciousOperation):
            anonyme_user.delete()
        with self.assertRaises(SuspiciousOperation):
            anonyme_user.save()


class LdapOK(CustomTestCase):
    @tag("low", "nologin")
    def test_ldapdev(self):
        # On teste si les utilisateurs de tests sont dans le ldap

        for user in test_user:
            userinfo = test_user[user]
            ldapuser = LdapUser.objects.get(login_inp_net=user)

            self.assertEqual(ldapuser.mail, userinfo["email"])
            self.assertEqual(ldapuser.inscrit_AE(), userinfo["AE"])
            self.assertEqual(ldapuser.promo, userinfo["year"])
            self.assertEqual(ldapuser.get_ecole().o, userinfo["ecole"])
            self.assertEqual(ldapuser.nom, userinfo["last_name"])
            self.assertEqual(ldapuser.prenom, userinfo["first_name"])
            for bureau in userinfo["bureau"]:
                self.assertIn(user, LdapClub.objects.get(cn=bureau).membres_bureau())
            for groupe in userinfo["membre"]:
                self.assertIn(user, LdapGroup.objects.get(cn=groupe).membres)
            if userinfo["admin"]:
                self.assertIn(user, LdapPosixGroup.objects.get(cn="admin-inp").membres)
            else:
                self.assertNotIn(user, LdapPosixGroup.objects.get(cn="admin-inp").membres)

    @tag("low", "nologin")
    def test_createLdapUser(self):
        for user in test_user:
            userinfo = test_user[user]
            ldapuser = LdapUser.objects.get(login_inp_net=user)
            localUser = User._default_manager.get_or_create(email=userinfo["email"])[0]
            localUser = AccountsCASBackend().configure_user(localUser)
            update_user(localUser)

            self.assertEqual(ldapuser.mail, localUser.email)
            self.assertEqual(ldapuser.inscrit_AE(), localUser.is_adherent_ae)
            self.assertEqual(ldapuser.promo, localUser.promo)
            self.assertEqual(ldapuser.get_ecole().o, localUser.ecole)
            self.assertEqual(ldapuser.nom, localUser.last_name)
            self.assertEqual(ldapuser.prenom, localUser.first_name)
            for bureau in userinfo["bureau"]:
                self.assertIn(localUser, Group.objects.get(cn=bureau).managers.all())
            for groupe in userinfo["membre"]:
                self.assertIn(localUser, Group.objects.get(cn=groupe).users.all())

    @tag("low", "nologin")
    def test_createLdapUserWithCasSignal(self):
        for user in test_user:
            userinfo = test_user[user]
            ldapuser = LdapUser.objects.get(login_inp_net=user)
            localUser = User._default_manager.get_or_create(email=userinfo["email"])[0]
            localUser = AccountsCASBackend().configure_user(localUser)
            cas_user_authenticated.send(
                sender=self,
                user=localUser,
                created=True,
                attributes={
                    "is_superuser": "True" if userinfo["admin"] else "False",
                    "uid": userinfo["email"],
                    "email": userinfo["email"],
                    "groups": str(userinfo["membre"]),
                },
                ticket="Test",
                service="http://billetterie.tests.inpt.fr/",
            )

            self.assertEqual(ldapuser.mail, localUser.email)
            self.assertEqual(ldapuser.inscrit_AE(), localUser.is_adherent_ae)
            self.assertEqual(ldapuser.promo, localUser.promo)
            self.assertEqual(ldapuser.get_ecole().o, localUser.ecole)
            self.assertEqual(ldapuser.nom, localUser.last_name)
            self.assertEqual(ldapuser.prenom, localUser.first_name)
            for bureau in userinfo["bureau"]:
                self.assertIn(localUser, Group.objects.get(cn=bureau).managers.all())
            for groupe in userinfo["membre"]:
                self.assertIn(localUser, Group.objects.get(cn=groupe).users.all())
            self.assertEqual(localUser.is_superuser, userinfo["admin"])

    @tag("low", "nologin")
    def test_createLdapUserWithOutLdap(self):
        # ce test permet de tester la fonction createLdapUser de Utils
        # et de s'assurer qu'elle fait effectue toutes les actions nécessaires
        for user in test_user:
            userinfo = test_user[user]
            ldapuser = LdapUser.objects.get(login_inp_net=user)
            localUser = createLdapUser(user)

            self.assertEqual(ldapuser.mail, localUser.email)
            self.assertEqual(ldapuser.inscrit_AE(), localUser.is_adherent_ae)
            self.assertEqual(ldapuser.promo, localUser.promo)
            self.assertEqual(ldapuser.get_ecole().o, localUser.ecole)
            self.assertEqual(ldapuser.nom, localUser.last_name)
            self.assertEqual(ldapuser.prenom, localUser.first_name)
            for bureau in userinfo["bureau"]:
                self.assertIn(localUser, Group.objects.get(cn=bureau).managers.all())
            for groupe in userinfo["membre"]:
                self.assertIn(localUser, Group.objects.get(cn=groupe).users.all())
            self.assertEqual(localUser.is_superuser, userinfo["admin"])

    @tag("low", "nologin")
    def test_createLdapUserInUtils(self):
        # ce test permet de tester la fonction createLdapUser de Utils
        # et de s'assurer qu'elle fait effectue toutes les actions nécessaires
        for user in test_user:
            userinfo = test_user[user]
            ldapuser = LdapUser.objects.get(login_inp_net=user)
            localUser = get_ldap_user(user)[0]

            self.assertEqual(ldapuser.mail, localUser.email)
            self.assertEqual(ldapuser.inscrit_AE(), localUser.is_adherent_ae)
            self.assertEqual(ldapuser.promo, localUser.promo)
            self.assertEqual(ldapuser.get_ecole().o, localUser.ecole)
            self.assertEqual(ldapuser.nom, localUser.last_name)
            self.assertEqual(ldapuser.prenom, localUser.first_name)
            for bureau in userinfo["bureau"]:
                self.assertIn(localUser, Group.objects.get(cn=bureau).managers.all())
            for groupe in userinfo["membre"]:
                self.assertIn(localUser, Group.objects.get(cn=groupe).users.all())
            self.assertEqual(localUser.is_superuser, userinfo["admin"])
            self.assertEqual(localUser, get_ldap_user(user)[0])  # fonction déterministe
        # change login_inp_net to generate error in import
        localUser = User.objects.get(login_inp_net="testone")
        localUser.login_inp_net = ""
        localUser.save()
        self.assertEqual(None, get_ldap_user("testone")[0])
        # test empty string
        self.assertEqual(None, get_ldap_user("")[0])
        # test duplicate login
        localUser = User.objects.get(login_inp_net="testthree")
        localUser.login_inp_net = "testwo"
        localUser.save()
        self.assertEqual(None, get_ldap_user("testwo")[0])
        # test not exists login
        self.assertEqual(None, get_ldap_user("nobodyhavethislogin")[0])
